@@ -9,12 +9,6 @@
 #ifndef __XML_EXSLTCONFIG_H__
 #define __XML_EXSLTCONFIG_H__
 
-#ifdef WIN32
-#include <win32config.h>
-#else
-#include "config.h"
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -24,21 +18,21 @@ extern "C" {
  *
  * the version string like "1.2.3"
  */
-#define LIBEXSLT_DOTTED_VERSION "1.0.5"
+#define LIBEXSLT_DOTTED_VERSION "1.0.6"
 
 /**
  * LIBEXSLT_VERSION:
  *
  * the version number: 1.2.3 value is 1002003
  */
-#define LIBEXSLT_VERSION 500
+#define LIBEXSLT_VERSION 600
 
 /**
  * LIBEXSLT_VERSION_STRING:
  *
  * the version number string, 1.2.3 value is "1002003"
  */
-#define LIBEXSLT_VERSION_STRING "500"
+#define LIBEXSLT_VERSION_STRING "600"
 
 /**
  * ATTRIBUTE_UNUSED:
