@@ -145,12 +145,12 @@ xsltFunctionNodeSet(xmlXPathParserContextPtr ctxt, int nargs){
     }
 }
 
-/*
- * Seems FreeBSD decided not to provide the timezone libc varaible
- * C.f bug report #59570 http://bugzilla.gnome.org/show_bug.cgi?id=59570
- */
-#ifndef __FreeBSD__
 
+/*
+ * Okay the following really seems unportable and since it's not
+ * part of any standard I'm not too ashamed to do this
+ */
+#if defined(linux) || defined(__sun)
 #if defined(HAVE_MKTIME) && defined(HAVE_LOCALTIME) && defined(HAVE_ASCTIME)
 #define WITH_LOCALTIME
 
@@ -196,7 +196,7 @@ xsltFunctionLocalTime(xmlXPathParserContextPtr ctxt, int nargs) {
     
     str = (char *) obj->stringval;
 
-    /* str = "$Date: 2001/09/11 10:47:56 $" */
+    /* str = "$Date: 2001/10/09 17:51:47 $" */
     memset(digits, 0, sizeof(digits));
     strncpy(digits, str+7, 4);
     field = strtol(digits, NULL, 10);
@@ -268,7 +268,7 @@ xsltFunctionLocalTime(xmlXPathParserContextPtr ctxt, int nargs) {
     valuePush(ctxt, xmlXPathNewString((xmlChar *)result));
 }
 #endif
-#endif /* __FreeBSD__ */
+#endif /* linux or sun */
 
 
 /**
